/*
 * STARSIDATypes.h - <see below>.
 *
 * Copyright (c) 2015 Zephyr Software LLC 
 *
 * This file is part of the STARS (STatic Analyzer for Reliability ans Security) infrastructure.
 * This file may be used and modified for non-commercial purposes as long as 
 * all copyright, permission, and nonwarranty notices are preserved.  
 * Redistribution is prohibited without prior written consent from Zephyr Software LLC 
 *
 * Please contact the authors for restrictions applying to commercial use.
 *
 * THIS SOURCE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Author: Clark L. Coleman
 * e-mail: clc@zephyr-software.com
 * URL   : http://www.zephyr-software.com/
 *
 */

#ifndef STARSIDATYPES_H
#define STARSIDATYPES_H 1

// STARSIDATypes.h
//
// This header defines the typedefs needed for obtaining program information
//  from the IDA Pro disassembler. It should only be included from IDA-specific
//  STARS modules.
//

#include <memory>

#if 0
#include <string>
#include <utility>
#include <list>
#include <vector>
#include <map>
#include <set>
#endif

#include <cstddef>
#include <cstdint>

#include <pro.h>   // for qstr... functions
// #include <ua.hpp>
// #include <fpro.h>  // for qf... functions
#include <bytes.hpp>   // for getFlags(), isCode(), IsHead(), get_item_end(), next_head(), get_flags_novalue()
#include <frame.hpp>   // for get_frame()
// #include <kernwin.hpp> // for msg()
// #include <lines.hpp>  // for tag_remove()
#include <name.hpp>  // for get_true_name()
#include <struct.hpp>  // for get_member_name()
#include <xref.hpp>  // for add_cref(), xrefblk_t

// Globals, typedefs and macros for STARS_IDA_INTERFACE only

#define SMP_isHead(flags) isHead(flags)
#define SMP_isCode(flags) isCode(flags)
#define SMP_get_item_end(addr) get_item_end(addr)
#define SMP_getFlags(addr) getFlags(addr)
// #if 1
#define SMP_get_frame(ptr_to_func_t) get_frame(ptr_to_func_t)
// #else
// #define SMP_get_frame(ptr_to_func_t) get_frame((func_t*)*dynamic_cast<STARS_IDA_Function_t*>(ptr_to_func_t))
// #endif
// #define SMP_get_member_name(mid, buf, bufsize) get_member_name(mid, buf, bufsize)
// #define SMP_get_true_name(from, ea, buf, bufsize) get_true_name(from, ea, buf, bufsize)
#define SMP_next_head(ea, maxea) next_head(ea, maxea)
#define SMP_get_flags_novalue(ea) get_flags_novalue(ea)

#define SMP_add_cref(from, to, type) add_cref(from, to, type)

struct SMP_xref_t {
	xrefblk_t CurrSMP_xref;
	STARS_ea_t GetFrom() { return CurrSMP_xref.from; };
	STARS_ea_t GetTo() { return CurrSMP_xref.to; };
	uchar GetIscode() { return CurrSMP_xref.iscode; };         // 1-is code reference; 0-is data reference
	uchar GetType() { return CurrSMP_xref.type; };           // type of the last retured reference (cref_t & dref_t)
	uchar GetUser() { return CurrSMP_xref.user; };           // 1-is used defined xref, 0-defined by ida
	bool SMP_first_from(STARS_ea_t from, int flags)   // get first reference from...
	{
		return xrefblk_t_first_from(&CurrSMP_xref, from, flags);
	}
	bool SMP_next_from(void)                   // get next reference from...
	{
		return xrefblk_t_next_from(&CurrSMP_xref);
	}
	bool SMP_first_to(STARS_ea_t to, int flags)      // get first reference to...
	{
		return xrefblk_t_first_to(&CurrSMP_xref, to, flags);
	}
	bool SMP_next_to(void)                     // get next reference to....
	{
		return xrefblk_t_next_to(&CurrSMP_xref);
	}
};

#endif
