#ifndef STARSInstructionID_h
#define STARSInstructionID_h

#include <cstdint>
#include <cassert>
#include <map>
#include <set>
#include <tuple>

#include "interfaces/STARSTypes.h"

class STARS_Instruction_t;

class STARS_InstructionID_t
{
	public:

		// constructor sets private data
		STARS_InstructionID_t(STARS_ea_t p_id, uint16_t FileNo = 0) : m_id(p_id), FileNumber(FileNo) {
		}
		
		const STARS_ea_t GetIDWithinFile(void) const { return m_id; };

		/* most of the time, though, we should map the ID to an insn */
		const STARS_Instruction_t* GetInstruction() { 
			const STARS_Instruction_t* ret = id_to_insn_map[m_id]; 
			assert(ret);
			return ret;
		};

		operator const STARS_Instruction_t* ()  { return GetInstruction(); }

		uint16_t GetFileNum(void) const { return FileNumber; };

		void AddIDToInsnMapping(const STARS_Instruction_t* to) const { id_to_insn_map[m_id] = to; } 

		void SetFileNum(uint16_t FileNum) { FileNumber = FileNum; };

		virtual void Dump();

		friend bool operator<(const STARS_InstructionID_t& l, const STARS_InstructionID_t& r) 
		{
			return std::tie(l.m_id, l.FileNumber) < std::tie(r.m_id, r.FileNumber); // keep the same order
		}


		bool HasCorrespondingInstructionIR() { return id_to_insn_map.find(m_id)!=id_to_insn_map.end(); }

	protected:

		STARS_ea_t m_id;
		uint16_t FileNumber; // for multi-file analysis, e.g. main exe plus shared objects
		static std::map<STARS_ea_t, const STARS_Instruction_t*> id_to_insn_map;

};

typedef std::set<STARS_InstructionID_t> STARS_InstructionID_Set_t;


#endif



